/*
 * main.c
 *
 *  Created on: 26.02.2018
 *  RS232 test program for VCSBC nano Z
 */

#include <stdio.h>
#include <vclib.h>
#include <vclinux.h>
#include <vcimgnet.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <termios.h>
#include <vc_env_console.h>
#include <i2c-dev.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#include <vcgpio.h>

#define DATASIZE 64

#define VERSION 1.0

int kbready();

I32 serialInitBus(I32 *serialHandle, struct termios *oldtio, struct termios *newtio);
I32 serialDeInitBus(I32 serialHandle, struct termios *oldtio);

/************************************************************************************************************************/

I32  main()
{
    I32         rc=0, err=0;
	I32         file_serial=-1;
	U8          rs232WriteData[DATASIZE] = "VisionComponents";
	U8          rs232ReadData[DATASIZE];
	char        nanoCode[256];
	char        nanoSerial[256];
	struct      termios oldtio, newtio;

	SVCEnvConsole  sVCEnvConsole;


	/////////////////////////////////////////////////////////////////////////////////////
	// Read license
	/////////////////////////////////////////////////////////////////////////////////////


	printf("\n\033[1;34m****************** Test program for VCSBC Z Board v%1.1f ******************\033[0m\n\n", VERSION);

	rc = fread_license("/dev/mtd3", "P/N", nanoCode, nanoSerial);
	if(rc<0){err=-2; goto end;}

	printf("\033[1;34m  Camera S/N:   %s\033[0m\n\n", nanoSerial);
	printf("\033[1;34m*************************************************************************\033[0m\n\n\n");



	/////////////////////////////////////////////////////////////////////////////////////
	// Init sequence
	/////////////////////////////////////////////////////////////////////////////////////

	// Init VCLIB
    rc =  vc_license_init("vclib");
    if(rc<0){err=-3; goto end;}

	// Disable serial shell
	system("systemctl stop serial-getty@ttyPS0.service");



	/////////////////////////////////////////////////////////////////////////////////////
	// RS232 test
	/////////////////////////////////////////////////////////////////////////////////////

	printf("\n\033[1;37m*************** START RS232 TEST... ***************\033[0m\n\n");

	// Open serial bus
	printf("  Open serial bus... ");
	rc = serialInitBus(&file_serial, &oldtio, &newtio);
	if(rc<0){printf("\nCould not open serial bus!\n"); err=-401; goto end;}
	printf("OK!\n");

	// Write something to RS232 interface, it should be echoed due to hardware loopback (adapter needed)
	printf("  Test RS232 interface... ");
	rc =  write(file_serial, rs232WriteData, 17);
	if(rc<0){printf("\nCould not write to serial interface!\n"); err=-402; goto end;}

	usleep(10000);

	// Read data
	rc =  read (file_serial, rs232ReadData, 17);
	if(rc<0){printf("\nCould not read from serial interface!\n"); err=-403; goto end;}

	// Print out read data
//	printf("(Sent: \"%s\" Read: \"%s\")... ", rs232WriteData, rs232ReadData);

	// Compare sent and read data
	if(strcmp((char*)rs232WriteData, (char*)rs232ReadData) == 0)
		printf("OK!\n");
	else
		{printf("Sent and received data are not identical!\n"); err=-404; goto end;}

  	// Close serial bus
	printf("  Close serial bus... ");
	rc = serialDeInitBus(file_serial, &oldtio);
	if(rc<0){printf("\nCould not close serial bus!\n"); err=-405; goto end;}
	printf("OK!\n");

	printf("\n\033[1;37m******************* RS232 TEST OK! *******************\033[0m\n\n\n");
	usleep(200000);



end:

	printf("\nEnd of program:\nrc = %d, err = %d\n\n\n", rc, err);

	if(err == 0)
	{
		printf("\033[1;32m*************** BOARD TEST SUCCESSFUL! ***************\033[0m\n");
	}
	else
	{
		printf("\033[1;31m*************** RS232 TEST NOT SUCCESSFUL! ***************\033[0m\n");

	}


//	vc_env_console_deinit(&sVCEnvConsole);

	// Reenable serial shell
	system("systemctl start serial-getty@ttyPS0.service");

    return(err);
}

/************************************************************************************************************************/

I32 serialInitBus(I32 *serialHandle, struct termios *oldtio, struct termios *newtio)
{
	I32 err=0, rc=0;

	// Open i2c bus
	*serialHandle = open("/dev/ttyPS0", O_RDWR|O_NONBLOCK);  // |O_NONBLOCK = non-blocking flag active!! Necessary to get error messages in our case
	if (*serialHandle < 0) {
//		printf("Failed to open the i2c bus!\n");
		err=-1;
	}

	rc =  tcgetattr(*serialHandle, oldtio); /* save current serial port settings */
//	bzero(newtio, sizeof(*newtio)); /* clear struct for new port settings */
	if(rc<0){err=-1001; goto end;}

	newtio = oldtio;

#if 0
	/*
	  BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
	  CRTSCTS : output hardware flow control (only used if the cable has
	            all necessary lines. See sect. 7 of Serial-HOWTO)
	  CS8     : 8n1 (8bit,no parity,1 stopbit)
	  CLOCAL  : local connection, no modem contol
	  CREAD   : enable receiving characters
	*/
//	 newtio->c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;  //CRTSCTS
	 newtio->c_cflag |= CREAD;

	 /*
	   IGNPAR  : ignore bytes with parity errors
	   ICRNL   : map CR to NL (otherwise a CR input on the other computer
	             will not terminate input)
	   otherwise make device raw (no other input processing)
	 */
	  newtio->c_iflag &= ~(ICRNL);

	 /*
	  Raw output.
	 */
//	  newtio->c_oflag = 0;

	 /*
	   ICANON  : enable canonical input
	   disable all echo functionality, and don't send signals to calling program
	 */
//	  newtio->c_lflag |= ICANON;


	/*
	  now clean the modem line and activate the settings for the port
	*/
//	tcflush(*serialHandle, TCIFLUSH);
	tcsetattr(*serialHandle, TCSANOW, newtio);
#endif


	 //
	 // Input flags - Turn off input processing
	 //
	 // convert break to null byte, no CR to NL translation,
	 // no NL to CR translation, don't mark parity errors or breaks
	 // no input parity check, don't strip high bit off,
	 // no XON/XOFF software flow control
	 //
	newtio->c_iflag &= ~(IGNBRK | BRKINT | ICRNL |
	                     INLCR | PARMRK | INPCK | ISTRIP | IXON);

	 //
	 // Output flags - Turn off output processing
	 //
	 // no CR to NL translation, no NL to CR-NL translation,
	 // no NL to CR translation, no column 0 CR suppression,
	 // no Ctrl-D suppression, no fill characters, no case mapping,
	 // no local output processing
	 //
	 // config.c_oflag &= ~(OCRNL | ONLCR | ONLRET |
	 //                     ONOCR | ONOEOT| OFILL | OLCUC | OPOST);
	newtio->c_oflag = 0;

	 //
	 // No line processing
	 //
	 // echo off, echo newline off, canonical mode off,
	 // extended input processing off, signal chars off
	 //
	newtio->c_lflag &= ~(ECHO | ECHONL | ICANON | IEXTEN | ISIG);

	 //
	 // Turn off character processing
	 //
	 // clear current char size mask, no parity checking,
	 // no output processing, force 8 bit input
	 //
	 newtio->c_cflag &= ~(CSIZE | PARENB);
	 newtio->c_cflag |= CS8;

	 //
	 // One input byte is enough to return from read()
	 // Inter-character timer off
	 //
	 newtio->c_cc[VMIN]  = 1;
	 newtio->c_cc[VTIME] = 0;

	 //
	 // Communication speed (simple version, using the predefined
	 // constants)
	 //
//	 if(cfsetispeed(&config, B9600) < 0 || cfsetospeed(&config, B9600) < 0) {
//	     ... error handling ...
//	 }

	 //
	 // Finally, apply the configuration
	 //
	 rc =  tcsetattr(*serialHandle, TCSAFLUSH, newtio);
	 if(rc<0){err=-1001; goto end;}

end:

	return(err);
}

/************************************************************************************************************************/

I32 serialDeInitBus(I32 serialHandle, struct termios *oldtio)
{
	I32 err=0;

#if 1
	/* restore the old port settings */
	tcsetattr(serialHandle,TCSANOW,oldtio);
#endif

	// Close i2c bus
	err = close(serialHandle);

	return(err);
}

/************************************************************************************************************************/
